@@ -4,22 +4,22 @@ import logging
 import os
 import shutil
 import subprocess
-import tempfile
 import textwrap
 import threading
 from logging import Logger
 from pathlib import Path
-from typing import TYPE_CHECKING, Any, Iterable, Mapping
+from typing import TYPE_CHECKING, Any, Iterable, Mapping, cast
 
-from pep517.wrappers import Pep517HookCaller
+from pyproject_hooks import BuildBackendHookCaller
 
 from pdm.compat import tomllib
 from pdm.exceptions import BuildError
+from pdm.models.environment import PrefixEnvironment
 from pdm.models.in_process import get_sys_config_paths
-from pdm.models.requirements import parse_requirement
+from pdm.models.requirements import Requirement, parse_requirement
 from pdm.models.working_set import WorkingSet
 from pdm.termui import logger
-from pdm.utils import create_tracked_tempdir, prepare_pip_source_args
+from pdm.utils import create_tracked_tempdir
 
 if TYPE_CHECKING:
     from pdm.models.environment import Environment
@@ -42,6 +42,7 @@ class LoggerWrapper(threading.Thread):
         self.fd_read, self.fd_write = os.pipe()
 
         self.start()
+        self._output_buffer: list[str] = []
 
     def fileno(self) -> int:
         return self.fd_write
@@ -57,12 +58,33 @@ class LoggerWrapper(threading.Thread):
 
     def _write(self, message: str) -> None:
         self.logger.log(self.level, message)
+        self._output_buffer.append(message)
+        if len(self._output_buffer) > 10:
+            self._output_buffer[:-10] = []
 
     def stop(self) -> None:
         os.close(self.fd_write)
         self.join()
 
 
+def build_error(e: subprocess.CalledProcessError) -> BuildError:
+    """Get a build error with meaningful error message
+    from the subprocess output.
+    """
+    output = cast("list[str]", e.output)
+    errors: list[str] = []
+    if output[-1].strip().startswith("ModuleNotFoundError"):
+        package = output[-1].strip().split()[-1]
+        errors.append(
+            f"Module {package} is missing, please make sure it is specified in the "
+            "'build-system.requires' section. If it is not possible, "
+            "add it to the project and use '--no-isolation' option."
+        )
+    errors.extend(["Showing the last 10 lines of the build output:", *output])
+    error_message = "\n".join(errors)
+    return BuildError(f"Build backend raised error: {error_message}")
+
+
 def log_subprocessor(
     cmd: list[str],
     cwd: str | Path | None = None,
@@ -81,11 +103,8 @@ def log_subprocessor(
             stderr=subprocess.STDOUT,
         )
     except subprocess.CalledProcessError as e:
-        raise BuildError(
-            f"Call command {cmd} return non-zero status({e.returncode}). "
-            "Make sure the package is PEP 517-compliant, or you can add "
-            "`--no-isolation` to the command."
-        ) from None
+        e.output = outstream._output_buffer
+        raise build_error(e) from None
     finally:
         outstream.stop()
 
@@ -141,7 +160,7 @@ class EnvBuilder:
     _overlay_envs: dict[str, str] = {}
 
     if TYPE_CHECKING:
-        _hook: Pep517HookCaller
+        _hook: BuildBackendHookCaller
         _requires: list[str]
         _prefix: _Prefix
 
@@ -190,7 +209,7 @@ class EnvBuilder:
 
     def init_build_system(self, build_system: dict[str, Any]) -> None:
         """Initialize the build system and requires list from the PEP 517 spec"""
-        self._hook = Pep517HookCaller(
+        self._hook = BuildBackendHookCaller(
             self.src_dir,
             build_system["build-backend"],
             backend_path=build_system.get("backend-path"),
@@ -206,9 +225,6 @@ class EnvBuilder:
             overlay=self.get_overlay_env(os.path.normcase(self.src_dir).rstrip("\\/")),
         )
 
-        if build_system["build-backend"].startswith("setuptools"):
-            self.ensure_setup_py()
-
     @property
     def _env_vars(self) -> dict[str, str]:
         paths = self._prefix.bin_dirs
@@ -244,7 +260,7 @@ class EnvBuilder:
             env.update(extra_environ)
         return log_subprocessor(cmd, cwd, extra_environ=env)
 
-    def check_requirements(self, reqs: Iterable[str]) -> Iterable[str]:
+    def check_requirements(self, reqs: Iterable[str]) -> Iterable[Requirement]:
         missing = set()
         conflicting = set()
         project_lib = self._env.get_paths()["purelib"]
@@ -262,7 +278,7 @@ class EnvBuilder:
                     )
                     continue
                 if parsed_req.identify() not in ws:
-                    missing.add(req)
+                    missing.add(parsed_req)
                 elif parsed_req.specifier and not parsed_req.specifier.contains(
                     ws[parsed_req.identify()].version, prereleases=True
                 ):
@@ -272,33 +288,14 @@ class EnvBuilder:
         return missing
 
     def install(self, requirements: Iterable[str], shared: bool = False) -> None:
-        missing = self.check_requirements(requirements)
+        from pdm.installers.core import install_requirements
+
+        missing = list(self.check_requirements(requirements))
         if not missing:
             return
         path = self._prefix.shared if shared else self._prefix.overlay
-
-        with tempfile.NamedTemporaryFile(
-            "w+", prefix="pdm-build-reqs-", suffix=".txt", delete=False
-        ) as req_file:
-            req_file.write(os.linesep.join(missing))
-            req_file.close()
-            cmd = self._env.pip_command + [
-                "install",
-                "--isolated",
-                "--ignore-installed",
-                "--prefix",
-                path,
-            ]
-            ca_certs = self._env.project.config.get("pypi.ca_certs")
-            if ca_certs is not None:
-                cmd.extend(["--cert", ca_certs])
-            client_cert = self._env.project.config.get("pypi.client_cert")
-            if client_cert is not None:
-                cmd.extend(["--client-cert", client_cert])
-            cmd.extend(prepare_pip_source_args(self._env.project.sources))
-            cmd.extend(["-r", req_file.name])
-            self.subprocess_runner(cmd, isolated=False)
-            os.unlink(req_file.name)
+        env = PrefixEnvironment(self._env.project, path)
+        install_requirements(missing, env)
 
         if shared:
             # The shared env is prepared and is safe to be cached now. This is to make
@@ -326,16 +323,3 @@ class EnvBuilder:
         return the absolute path of the built result.
         """
         raise NotImplementedError("Should be implemented in subclass")
-
-    def ensure_setup_py(self) -> str:
-        from pdm.pep517.base import Builder
-        from pdm.project.metadata import MutableMetadata
-
-        builder = Builder(self.src_dir)
-        project_file = os.path.join(self.src_dir, "pyproject.toml")
-        if os.path.exists(project_file):
-            try:
-                builder._meta = MutableMetadata.from_file(project_file)
-            except ValueError:
-                builder._meta = None
-        return builder.ensure_setup_py().as_posix()
