@@ -334,6 +334,15 @@ def normalize_name(name: str, lowercase: bool = True) -> str:
     return name.lower() if lowercase else name
 
 
+def comparable_version(version: str) -> Version:
+    """Normalize a version to make it valid in a specifier."""
+    parsed = Version(version)
+    if parsed.local is not None:
+        # strip the local part
+        parsed._version = parsed._version._replace(local=None)
+    return parsed
+
+
 def is_egg_link(dist: Distribution) -> bool:
     """Check if the distribution is an egg-link install"""
     return getattr(dist, "link_file", None) is not None
@@ -396,10 +405,10 @@ def deprecation_warning(
     """Show a deprecation warning with the given message and raise an error
     after a specified version.
     """
-    from pdm.__version__ import parsed_version
+    from pdm.__version__ import __version__
 
-    if raise_since is not None and parsed_version:
-        if parsed_version >= Version(raise_since):
+    if raise_since is not None:
+        if Version(__version__) >= Version(raise_since):
             raise DeprecationWarning(message)
     warnings.warn(message, DeprecationWarning, stacklevel=stacklevel + 1)
 
